/**
 ** infusion - The Desktop Communications Revoution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: cxdaemon.o
 ** Date: 2001-03-08
 ** Last Revision: 2001-03-08
 ** Description: CxDaemon interface.
 ** CVS: $Id: cxdaemontray.cpp,v 1.4 2001/06/12 15:03:17 brian Exp $
 **/
#include	<unistd.h>
#include	<iostream.h>
#include	<qtimer.h>
#include	<qdatetime.h>
#include	<qstring.h>
#include	<qtooltip.h>
#include	<qdrawutil.h>
#include	<qbitmap.h>
#include	<kapp.h>
#include	<kpopupmenu.h>
#include	<kcmdlineargs.h>
#include	<kpixmapeffect.h>
#include	<kdebug.h>
#include	<kiconeffect.h>
#include	<klocale.h>
#include	<kglobal.h>
#include	<kconfig.h>
#include	<ksimpleconfig.h>
#include	<kstyle.h>
#include	<kiconloader.h>
#include	<kprocess.h>
#include	<kmessagebox.h>
#include	<knotifyclient.h>
#include	<CxClient.h>
#include	"cxdaemon.h"
#include	"wndabout.h"
#include	"wndbuddylist.h"
#include	"wndprefs.h"
#include	"cxconnection.h"
#include	"koevent.h"

/**
 ** CxDaemonTray::CxDaemonTray(): CONSTRUCTOR.
 **/
		CxDaemonTray::CxDaemonTray(QObject *parent, const char *name)
		:KSystemTray( NULL, name ) {

	if(parent) ;

	emit initIcons();
	emit setIconOffline();
	emit initConnection();
 
	itemId = contextMenu()->insertItem(i18n("&Online"), this, SLOT(toggleAlarmsEnabled()));
	contextMenu()->setItemChecked(itemId, FALSE);
	refreshId = contextMenu()->insertItem( i18n("&Refresh"), this, SLOT( syncLists() ));
	contextMenu()->insertItem(i18n("&Buddy List..."), this, SLOT(raiseBuddyDlg()));
	contextMenu()->setItemEnabled( refreshId, FALSE );
	contextMenu()->insertItem(i18n("&Preferences..."), this, SLOT(raisePrefsDlg()));
	contextMenu()->insertItem(i18n("&About..."), this, SLOT(raiseAboutDlg()));
	addToolTip( i18n("Offline") );

	w_buddyList = 0L;
	QTimer::singleShot( 230, this, SLOT(goOnline()));
}

/**
 ** CxDaemonTray::
 **/
		CxDaemonTray::~CxDaemonTray() {

	delete g_cxConxion;
}

/**
 ** CxDaemonTray::initConnection(): Initialize the Citadel/UX connection object.
 **/
void		CxDaemonTray::initConnection() {
QString		tmp;
KConfig		*kc = KGlobal::config();

	g_cxConxion = new CxConnection;

	kc->setGroup("Account");
	tmp = kc->readEntry( "Host" );
	cout << "Host: " << tmp << endl;
	if(!tmp.isNull()) g_cxConxion->setHostName( tmp );
	tmp = kc->readEntry( "Username" );
	cout << "Username: " << tmp << endl;
	if(!tmp.isNull()) g_cxConxion->setUserName( tmp );
	tmp = kc->readEntry( "Password" );
	cout << "Password: " << tmp << endl;
	if(!tmp.isNull()) g_cxConxion->setPassword( tmp );

	connect(g_cxConxion, SIGNAL(error(const QString&)), this, SLOT(errorMsg(const QString&)));
	connect(g_cxConxion, SIGNAL(comingOnline()), this, SLOT(setIconWorking()));
	connect(g_cxConxion, SIGNAL(goingOffline()), this, SLOT(setIconOffline()));
	connect(g_cxConxion, SIGNAL(amOnline()), this, SLOT(setIconOnline()));
	connect(g_cxConxion, SIGNAL(updatingStart()), this, SLOT(setIconWorking()));
	connect(g_cxConxion, SIGNAL(updatingFinish()), this, SLOT(setIconNormal()));
}

/**
 ** CxDaemonTray::initIcons(): Load and initialize the icons.
 **/
void		CxDaemonTray::initIcons() {

	KGlobal::iconLoader()->addAppDir("infusion");
	pix1 = BarIcon(QString("infusion"));
	pix2 = BarIcon(QString("infusion"));
	pix3 = BarIcon(QString("connect_no"));

	QImage img = pix2.convertToImage();
	KIconEffect *mpEffect = new KIconEffect;
	img = mpEffect->apply( img, KIconEffect::NoEffect, 1.0, QColor(144,128,248), true );
	pix2.convertFromImage( img );

	if (pix1.isNull() || pix2.isNull()) {
		KMessageBox::sorry(this, i18n("Can't load docking tray icon!"),
				i18n("Alarm Monitor Error"));
	}

}

/**
 ** CxDaemonTray::syncLists(): Instruct the CxConnection object to refresh any and all lists
 ** that it is keeping track of.  This also flushes the miscellaneous data caches which exist
 ** for the convenience of the User.
 **/
void		CxDaemonTray::syncLists() {

	emit g_cxConxion->sync();
}

/**
 ** CxDaemonTray::autoSyncLists(): Automated hook for syncLists().  This should execute
 ** repeatedly to keep CxDaemon up-to-date.
 **/
void		CxDaemonTray::autoSyncLists() {

	/**
	 ** If we are NOT online, break this nasty cycle.
	 **/
	if(!g_cxConxion->isOnline()) return;

	emit syncLists();
	QTimer::singleShot( 100000, this, SLOT(autoSyncLists()));
}

/**
 ** CxDaemonTray::errorMsg(): Display an error dialog.  This generally means that the
 ** Infusion Daemon was unable to connect to the server, or some other extraneous event
 ** hath occurred.
 **/
void		CxDaemonTray::errorMsg( const QString& err ) {

	KMessageBox::sorry(this,
                                QString("An error has occurred:\n")+
                                err,
                                QString("Error"),
                                false
                        );
}

/**
 ** CxDaemonTray::
 **/
bool		CxDaemonTray::alarmsOn() {
	return( false );
}

/**
 ** CxDaemonTray::
 **/
void		CxDaemonTray::addToolTip(const QString &msg) {

	QString txt = i18n("Infusion Daemon");
	if (!msg.isEmpty()) txt += "\n" + msg;
	QToolTip::add(this,txt);
}

/**
 ** CxDaemonTray::
 **/
void		CxDaemonTray::mousePressEvent(QMouseEvent *e) {

	if (e->button() == LeftButton) {

		emit launchInfusion();

	} else {
		KSystemTray::mousePressEvent(e);
	}
}

/**
 ** CxDaemonTray::launchInfusion(): Launch the master "Infusion" process.
 **/
void		CxDaemonTray::launchInfusion() {

	KProcess proc;
	proc << "infusion";
	proc.start(KProcess::DontCare);
}

/**
 ** CxDaemonTray::
 **/
void		CxDaemonTray::closeEvent(QCloseEvent *e) {

	if(e); 
	kapp->quit();
}

/**
 ** CxDaemonTray::toggleAlarmsEnabled():
 **/
void		CxDaemonTray::toggleAlarmsEnabled() {

	if(contextMenu()->isItemChecked(itemId)) {
		goOffline();
	} else {
		goOnline();
	}
}

/**
 ** CxDaemonTray::reapBuddyDlg(): Reap the Buddy List window.
 **/
void		CxDaemonTray::reapBuddyDlg() {

	cerr << "CxDaemonTray::reapBuddyDlg()" << endl;
	w_buddyList = 0L;
}

/**
 ** CxDaemonTray::raiseBuddyDlg(): Open up the Buddy List window.
 **/
void		CxDaemonTray::raiseBuddyDlg() {
QList<QString>	list;

	cerr << "CxDaemonTray::raiseBuddyDlg()" << endl;
	if(!w_buddyList) {
		w_buddyList = new CxWndBuddylist();
		connect( w_buddyList, SIGNAL(closing()), this, SLOT(reapBuddyDlg()));
		connect( w_buddyList, SIGNAL(openChatWindow( const QString& )), 
			this, SLOT(openChatWin( const QString& )));
	}

	list = g_cxConxion->getOnlineUsers();
	w_buddyList->slotPopulateOnline( list );
	list.clear();

	list = g_cxConxion->getAllUsers();
	w_buddyList->slotPopulateUsers( list );
	list.clear();

	w_buddyList->show();
}

/**
 ** CxDaemonTray::openChatWin(): Start chatting with a user.
 **/
void		CxDaemonTray::openChatWin( const QString& userName ) {
CxWndChat	*win;

	cerr << "CxDaemonTray::openChatWin(): " << userName << endl;
	win = &chatWin( userName );
	win->show();
}

/**
 ** CxDaemonTray::closeChatWin(): Start chatting with a user.
 **/
void		CxDaemonTray::closeChatWin( const QString& userName ) {

	cerr << "CxDaemonTray::closeChatWin(): " << userName << endl;
	if( w_chatWindows[userName] ) {
		delete w_chatWindows[userName];
		w_chatWindows.remove(userName);
	}
}

/**
 ** CxDaemonTray::chatWin(): Return a pointer to a fresh new chat window, or to an
 ** existing window.
 **/
CxWndChat	&CxDaemonTray::chatWin( const QString& userName ) {
CxWndChat	*nu;

	cerr << "CxDaemonTray::chatWin(): " << userName << endl;
	nu = w_chatWindows[userName];
	if(!nu) {
		cerr << "- new CxWndChat()" << endl;
		nu = new CxWndChat( userName );
		cerr << "- setUserName()" << endl;
		nu->setUserName( g_cxConxion->getUserName() );
		cerr << "- connect()" << endl;
		connect(nu, SIGNAL(closing(const QString &)), this, SLOT(closeChatWin(const QString &)) );
		connect(nu, SIGNAL(sendMessage(const QString &, const QString &)), g_cxConxion, SLOT(sendExpressMsg(const QString &, const QString &)) );
		connect(g_cxConxion, SIGNAL(expMesgFailed()), nu, SLOT(userIsOffline()));
		cerr << "- w_chatWindows.insert()" << endl;
		if(!w_chatWindows[userName]) w_chatWindows.insert(userName, nu);
	}
	return(*nu);
}

/**
 ** CxDaemonTray::raisePrefsDlg(): Open up the Infusion Daemon preferences.
 **/
void		CxDaemonTray::raisePrefsDlg() {

	CxWndPrefs *wndPrefs = new CxWndPrefs( this, 0 );
	wndPrefs->show();
}

/**
 ** CxDaemonTray::raiseAboutDlg(): Open up an About dialog for this program.
 **/
void		CxDaemonTray::raiseAboutDlg() {

	CxWndAbout *wndAbout = new CxWndAbout( kapp, 0 );
	wndAbout->show();
}

/**
 ** CxDaemonTray::goOffline(): Disconnect from server.
 **/
void		CxDaemonTray::goOffline() {

	emit g_cxConxion->citDisconnect();
}

/**
 ** CxDaemonTray::goOnline(): Connect to the configured server.
 **/
void		CxDaemonTray::goOnline() {

	emit g_cxConxion->citConnect();
}

/**
 ** CxDaemonTray::getFloorCache(): Return a reference to the current
 ** floor cache.
 **/
CXLIST		*CxDaemonTray::getFloorCache() {

	return( (g_cxConxion->floorCache()) );
}

/**
 ** CxDaemonTray::setIcon(): Set the active icon.
 **/
void		CxDaemonTray::setIcon( CurrentState state ) {

	switch( state ) {

		case( CxDaemonTray::Online ):
			setPixmap( pix1 );
			break;

		case( CxDaemonTray::Offline ):
			setPixmap( pix2 );
			break;

		case( CxDaemonTray::Working ):
			setPixmap( pix3 );
			break;

		default:
			setPixmap( pix2 );
			break;
	}
}

/**
 ** CxDaemonTray::setIconOnline(): Make the online icon active.
 **/
void		CxDaemonTray::setIconOnline() {

	cerr << "CxDaemonTray::setIconOnline()" << endl;
	emit setIcon( CxDaemonTray::Online );
	cerr << "CxDaemonTray::setIconOnline(): addToolTip()" << endl;
	emit addToolTip( i18n("Online - ")+g_cxConxion->getHostName() );
//	cerr << "contextMenu()->setItemEnabled()" << endl;
//	contextMenu()->setItemEnabled( refreshId, TRUE );
	cerr << "CxDaemonTray::setIconOnline(): autoSyncLists()" << endl;
	emit autoSyncLists();
}

/**
 ** CxDaemonTray::setIconNormal(): Make the online icon active, without throttling CxDaemon.
 **/
void		CxDaemonTray::setIconNormal() {

	if(!g_isWorking) return;
	g_isWorking = false;
	emit addToolTip( i18n("Online - ")+g_cxConxion->getHostName() );
	emit setIcon( CxDaemonTray::Online );
	contextMenu()->setItemEnabled( refreshId, TRUE );
}

/**
 ** CxDaemonTray::setIconWorking(): Make the online icon active.
 **/
void		CxDaemonTray::setIconWorking() {

	if(g_isWorking) return;
	g_isWorking = true;
	emit setIcon( CxDaemonTray::Working );
	emit addToolTip( i18n("Working... - ")+g_cxConxion->getHostName() );
	contextMenu()->setItemEnabled( refreshId, FALSE );
}

/**
 ** CxDaemonTray::setIconOffline(): Make the offline icon active.
 **/
void		CxDaemonTray::setIconOffline() {

	emit setIcon( CxDaemonTray::Offline );
	emit addToolTip( i18n("Offline") );
	contextMenu()->setItemEnabled( refreshId, FALSE );
}
