#include	<iostream.h>
#include	<qbuttongroup.h>
#include	<qgroupbox.h>
#include	<qlineedit.h>
#include	<qcombobox.h>
#include	<qlayout.h>
#include	<qlabel.h>
#include	<qvbox.h>
#include	<qwidget.h>
#include	<kdialogbase.h>
#include	<klocale.h>
#include	<kconfig.h>
#include	<kglobal.h>
#include	<kinstance.h>
#include	<kiconloader.h>
#include	<kglobalsettings.h>
#include	<ktabctl.h>
#include	"wndprefs.h"

/**
 ** CxWndPrefs::CxWndPrefs(): CONSTRUCTOR. Preferences dialog.  Hints taken
 ** from KMail.
 **/
		CxWndPrefs::CxWndPrefs(QWidget *parent, const char *name)
		:KDialogBase( IconList, i18n("Preferences"), Help|Default|Apply|Ok|Cancel, Ok, parent,
				name, true, true ) {

	setHelp( "infusion/configure.html", QString::null );
	setIconListAllVisible( true );
	enableButton( Default, false );

	connect(this, SIGNAL(applyClicked()), this, SLOT(saveSettings()));
	connect(this, SIGNAL(okClicked()), this, SLOT(saveAndExit()));

	createTabIdentity( this );
	mpConfig = KGlobal::config();

	loadSettings();
}

/**
 ** CxWndPrefs::createTabIdentity(): Create the "Identity" pane.
 **/
void		CxWndPrefs::createTabIdentity( QWidget *parent ) {

	QFrame *page = addPage( i18n("Identity"), i18n("Personal Information"),
		KGlobal::instance()->iconLoader()->loadIcon("identity", KIcon::NoGroup, KIcon::SizeMedium ));
	QVBoxLayout *topLevel = new QVBoxLayout( page, 0, spacingHint() );
	QGridLayout *layout = new QGridLayout( topLevel, 12, 3 );
	layout->addColSpacing(1, fontMetrics().maxWidth()*15 );
	layout->addRowSpacing(6, spacingHint() );
	layout->setRowStretch( 11, 10 );
	layout->setColStretch( 1, 10 );

	QLabel *label = new QLabel( i18n("Identity:"), page );
	layout->addWidget( label, 0, 0 );
	
	label = new QLabel( i18n("Citadel Host:"), page );
	layout->addWidget( label, 1, 0 );

	t_Host = new QComboBox( true, page );
	t_Host->setInsertionPolicy( QComboBox::AtTop );
	t_Host->insertItem( SmallIcon("host"), i18n("uncensored.citadel.org") );
	t_Host->insertItem( SmallIcon("host"), i18n("pixel.citadel.org") );
	layout->addMultiCellWidget( t_Host, 1, 1, 1, 2 );

	label = new QLabel( i18n("Real Name:"), page );
	layout->addWidget( label, 2, 0 );

	t_fullName = new QLineEdit( page );
	layout->addWidget( t_fullName, 2, 1 );

	label = new QLabel( i18n("Username:"), page );
	layout->addWidget( label, 3, 0 );

	t_Username = new QLineEdit( page );
	layout->addWidget( t_Username, 3, 1 );

	label = new QLabel( i18n("Password:"), page );
	layout->addWidget( label, 4, 0 );

	t_Password = new QLineEdit( page );
	t_Password->setEchoMode( QLineEdit::Password );
	layout->addWidget( t_Password, 4, 1 );
}

/**
 ** CxWndPrefs::loadSettings(): Load the current settings from disk.
 **/
void		CxWndPrefs::loadSettings() {
QString		tmp;

	cout << "CxWndPrefs::loadSettings(): Current settings:" << endl;

	mpConfig->setGroup("Account");
	tmp = mpConfig->readEntry("Host");
	if(tmp) {
		t_Host->insertItem( SmallIcon("host"), tmp );
		t_Host->setCurrentItem( 0 );
	}

	tmp = mpConfig->readEntry("Username");
	if(!tmp.isNull()) t_Username->setText(tmp);

	tmp = mpConfig->readEntry("Password");
	if(!tmp.isNull()) t_Password->setText(tmp);

	tmp = mpConfig->readEntry("FullName");
	if(!tmp.isNull()) t_fullName->setText(tmp);

	cout << "CxWndPrefs::loadSettings(): FullName: " << t_fullName->text() << endl;
	cout << "CxWndPrefs::loadSettings(): Username: " << t_Username->text() << endl;
	cout << "CxWndPrefs::loadSettings(): Password: " << t_Password->text() << endl;
}

/**
 ** CxWndPrefs::saveSettings(): Save the current settings.
 **/
void		CxWndPrefs::saveSettings() {

	cout << "CxWndPrefs::saveSettings(): Current settings:" << endl;
	cout << "CxWndPrefs::saveSettings(): FullName: " << t_fullName->text() << endl;
	cout << "CxWndPrefs::saveSettings(): Username: " << t_Username->text() << endl;
	cout << "CxWndPrefs::saveSettings(): Password: " << t_Password->text() << endl;

	mpConfig->setGroup("Account");
	mpConfig->writeEntry("Host", t_Host->currentText(), true, true );
	mpConfig->writeEntry("FullName", t_fullName->text(), true, true );
	mpConfig->writeEntry("Username", t_Username->text(), true, true );
	mpConfig->writeEntry("Password", t_Password->text(), true, true );
	mpConfig->sync();
}

/**
 ** CxWndPrefs::saveAndExit(): Save settings and exit.
 **/
void		CxWndPrefs::saveAndExit() {

	cout << "CxWndPrefs::saveAndExit(): Saving..." << endl;
	emit saveSettings();

	cout << "CxWndPrefs::saveAndExit(): Exiting..." << endl;
	emit close();
}
