/**
 ** infusion - The Desktop Communications Revolution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: modules/ui/kde2/objnotesview.o
 ** Date: 2000-12-18
 ** Last Revision: 2000-12-18
 ** Description: Notes View component.
 ** CVS: $Id: objnotesview.cpp,v 1.5 2001/05/16 22:40:44 brian Exp $
 **/
#define         __CX_SOURCEFILE__
#include        <unistd.h>
#include        <stdlib.h>
#include        <stdio.h>
#include        <string.h>
#include        <iostream.h>
#include        <qlayout.h>
#include        <qevent.h>
#include        <qhbox.h>
#include        <qsplitter.h>
#include        <kapp.h>
#include        <klocale.h>
#include        <kglobal.h>
#include        <kmessagebox.h>
#include        <kiconview.h>
#include        <kiconloader.h>
#include        <CxClient.h>
#include	"objnotesview.h"
#include	"objnote.h"
#include	"wndmain.h"

extern CxWndMain *wndMain;

/**
 ** CxObjNotesView::CxObjNotesView(): Generate a new IconView containing notes.
 **/
		CxObjNotesView::CxObjNotesView(QWidget *parent, const char *name): 
		KIconView(parent,name) {

	setAutoArrange( true );
	connect(this,SIGNAL(executed(QIconViewItem *)), this, SLOT(slotSelected(QIconViewItem *)));
	connect( this, SIGNAL(rightButtonClicked( QIconViewItem*, const QPoint& )), this, SLOT(rmbClicked(QIconViewItem *, const QPoint &)) );

	m_NoItem = new QPopupMenu();
	m_NoItem->insertItem(SmallIcon(QString("knotes")), QString("New &Note..."), this, SLOT(slotNewNote()));

	m_Item = new QPopupMenu();
	m_Item->insertItem(SmallIcon(QString("knotes")), QString("&Open"), this, SLOT(slotSelected( (QIconViewItem *)NULL )));
}

/**
 ** CxObjNotesView::refreshList(): Populate the notes list from the server's database.  This process
 ** reads the notes stored in the "My Notes" room, and "_GLOB_NOTES_"
 **/
void		CxObjNotesView::refreshList() {
char		*s, *c;
QString		qs, index;
QIconViewItem	*i;
CXLIST		list, l;
MESGINFO	msg;
ROOMINFO	*ri;

	// You _must_ do this.  Otherwise, program go bewm!
	msg.body = 0;

	// Go to the user's PERSONAL notes room.
	ri = CxRmGoto(wndMain->getHndl(), "My Notes",0);
	if(ri) {
		this->clear();
		Notes.clear();

		// Unnecessary data.
		free(ri);

		// Fetch the room index.
		list = CxMsList(wndMain->getHndl(), MSGS_ALL, 0);
		l = list;

		// Load all of the notes from "My Notes".
		while(l) {

			// If the note exists, add its body.
			if(!CxMsLoad(wndMain->getHndl(), l->data, 1, &msg)) {
				s = msg.body;
				qs = QString(s);

				c = s;
				while(*(c)) {
					if(((*(c)=='.') || (*(c)=='?')) 
						&& 
						((*(c+1)==' ') || !(*(c+1)))) {
						*(c+1) = 0;
					}
					c++;
				}

				index = QString(s);
				Notes.insert(index, new QString(qs));

				delete(qs);

				i = new QIconViewItem(this, msg.body, DesktopIcon(QString("knotes")));
				insertItem(i);
				free(msg.body);
				msg.body = 0;	// Do this or DIE.
			}
			l = l->next;
		}

		// Eliminate the room list.
		list = CxLlFlush(list);

		// Go back to where we belong.	
		ri = CxRmGoto(wndMain->getHndl(), wndMain->getRoomName(),0);

		free(ri);

	} else {

		ROOMINFO ri;
		strcpy(ri.name, "My Notes");
		ri.mode = 4;
		ri.floor_id = 0;

		if(CxRmCreate(wndMain->getHndl(), ri)) {
			i = new QIconViewItem(this,"I'm sorry, but your account is missing the \"My Notes\" folder.  Please contact the system administrator for assistance.", DesktopIcon(QString("knotes")));
			insertItem(i);
		} else {
			i = new QIconViewItem(this,"A new 'My Notes' folder has been created for you.", DesktopIcon(QString("knotes")));
			insertItem(i);
		}
	}
}

/**
 ** CxObjNotesView::slotNewNote(): Pop up a blank notes window.
 **/
void		CxObjNotesView::slotNewNote() {
	CxWndNote *newNote = new CxWndNote(NULL, QString::null);
	newNote->show();
}

/**
 ** CxObjNotesView::slotSelected(): Handle a selected (executed) object.
 **/
void		CxObjNotesView::slotSelected(QIconViewItem *qi) {
QString		ci;

	if(!qi) qi = currentItem();
	if(Notes[qi->text()]) {
		ci = (Notes[qi->text()])->copy();
		CxWndNote *n = new CxWndNote(NULL, ci);
		n->show();
	} else {
		KMessageBox::error(this, i18n("I'm sorry, but I could not load the note you requested."), i18n("Error Loading Note"),false);
	}
}

/**
 ** CxObjNotesView::slotSelected(): Handle a selected (executed) object.
 **/
void		CxObjNotesView::slotSelected() {
QString		ci;
QIconViewItem	*qi = 0;

	qi = currentItem();
	if(Notes[qi->text()]) {
		ci = (Notes[qi->text()])->copy();
		CxWndNote *n = new CxWndNote(NULL, ci);
		n->show();
	} else {
		KMessageBox::error(this, i18n("I'm sorry, but I could not load the note you requested."), 
			i18n("Error Loading Note"),false);
	}
}

/**
 ** CxObjNotesView::rmbClicked(): Context-sensitive Menu
 **/
void		CxObjNotesView::rmbClicked(QIconViewItem *item, const QPoint& pt) {

	if(item) {
		emit selectionChanged(item);
		m_Item->popup(pt);
	} else {
		m_NoItem->popup(pt);
	}
}

/**
 ** CxObjNotesView::viewLarge(): Adjust the icon view properties..
 **
CxObjNotesView::viewLarge() {
}
 **/
